import sys
import traceback

from celery import states
from celery.schedules import crontab

from concentrator import settings
from concentrator.smev3_v321.service_types import kinder_order
from kinder.core.async_tasks.tasks import AsyncTask
from kinder.webservice.smev3.tasks.consumer import ReceivingAsConsumerTask

from ..models import OrderRequest
from .helpers import OrderRequestHelper
from .helpers import UpdateRequestHelper


class OrderRequestSendingTask(AsyncTask):
    """
    Формирование и отправка запроса OrderRequest СМЭВ 3
    с блоком CreateOrderRequest.
    """

    description = 'Формирование и отправка запроса OrderRequest СМЭВ 3.'

    def run(self, *args, **kwargs):
        super().run(*args, **kwargs)

        self.set_progress(
            'Запущено формирование и отправка запроса CreateOrderRequest.'
        )

        try:
            order_request_helper = OrderRequestHelper(**kwargs)
            result = order_request_helper.send_request()
            order_request_helper.log(result)
        except Exception:
            error_message = (
                '\n'.join(traceback.format_exception(*sys.exc_info())))
            OrderRequestHelper.log(None, result=error_message)

            self.set_progress(
                progress='Ошибка',
                task_state=states.FAILURE,
                values={'Ошибка': error_message}
            )
        else:
            self.set_progress(
                progress='Запрос CreateOrderRequest успешно отправлен.',
                task_state=states.SUCCESS
            )

        return self.state


class OrderRequestReceivingTask(ReceivingAsConsumerTask):
    """Задача обработки полученных ответов на запрос OrderRequest СМЭВ 3."""

    description = (
        'Задача обработки полученных ответов на запрос OrderRequest СМЭВ 3')

    model = OrderRequest
    message_type = 'OrderRequest'

    run_every = crontab(
        minute=settings.SMEV3_FORM_DATA_TASK_EVERY_MINUTE,
        hour=settings.SMEV3_FORM_DATA_TASK_EVERY_HOUR
    )

    def _process(self, request):
        pass

    def process_message(self, message):
        super().process_message(message)
        message.request.order_id = kinder_order.parseString(
            message.body).CreateOrderResponse.orderId
        message.request.save()

    def validate_response(self, response):
        return True


class UpdateOrderRequestSendingTask(AsyncTask):
    """
    Формирование и отправка запроса OrderRequest СМЭВ 3
    с блоком UpdateOrderRequest.
    """

    description = 'Формирование и отправка запроса UpdateOrderRequest СМЭВ 3.'

    def run(self, *args, **kwargs):
        super().run(*args, **kwargs)

        self.set_progress(
            'Запущено формирование и отправка запроса UpdateOrderRequest.'
        )

        try:
            update_request_helper = UpdateRequestHelper(**kwargs)
            result = update_request_helper.send_request()
            update_request_helper.log(result)
        except Exception:
            error_message = (
                '\n'.join(traceback.format_exception(*sys.exc_info())))
            UpdateRequestHelper.log(None, result=error_message)

            self.set_progress(
                progress='Ошибка',
                task_state=states.FAILURE,
                values={'Ошибка': error_message}
            )
        else:
            self.set_progress(
                progress='Запрос UpdateOrderRequest успешно отправлен.',
                task_state=states.SUCCESS
            )

        return self.state
